package com.xhkjedu.sexam.service.report;

import com.xhkjedu.sexam.mapper.exam.EBaseMapper;
import com.xhkjedu.sexam.mapper.exam.EClassMapper;
import com.xhkjedu.sexam.mapper.exam.ESubjectMapper;
import com.xhkjedu.sexam.mapper.paper.EPaperFileMapper;
import com.xhkjedu.sexam.mapper.paper.EPaperMapper;
import com.xhkjedu.sexam.mapper.reportstu.ERstudentMapper;
import com.xhkjedu.utils.N_Utils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author ywx
 * @classname EReportStudentService
 * @description 学生报告
 * @date 2022/8/8 9:47
 **/
@Service
public class EReportStudentService {
    @Resource
    private EBaseMapper eBaseMapper;
    @Resource
    private ESubjectMapper eSubjectMapper;
    @Resource
    private EClassMapper eClassMapper;
    @Resource
    private ERstudentMapper eRstudentMapper;
    @Resource
    private EPaperMapper ePaperMapper;
    @Resource
    private EPaperFileMapper ePaperFileMapper;

    /**
     * @Description 获取考试基本信息（科目、班级、班级下学生）
     * @Param [examid]
     * @Return java.util.Map
     * @Author wn
     * @Date 2022/8/25 10:25
     **/
    public Map getExamSubjectClass(Integer examid) {
        Map map = eBaseMapper.findById(examid);
        List<Map> subjects = eSubjectMapper.listSubjectAndPnum(examid);
        map.put("subjects", subjects);
        List<Map> classes = eClassMapper.listExamClassAndStu(examid);
        map.put("classes", classes);
        return map;
    }

    /**
     * @Description 报告概况--获取学生各科成绩单
     * @Param [examid]
     * @Return java.util.Map
     * @Author wn
     * @Date 2022/8/25 14:23
     **/
    public Map listStudentSubjectScoreByExamid(Integer examid, Integer studentid) {
        List<Map> list = eRstudentMapper.getStudentScoreByExamid(examid, studentid);
        Map rtnmap = new HashMap();
        rtnmap.put("subjects", list);
        if (list != null && list.size() > 0) {
            Map map1 = list.get(0);
            Integer classid = Integer.parseInt(map1.get("classid").toString());
            Map gcnum = eRstudentMapper.getStudentNumByExamid(examid, classid);
            rtnmap.put("classnum", gcnum.get("classnum").toString());
            rtnmap.put("gradenum", gcnum.get("gradenum").toString());
        }
        return rtnmap;
    }

    /**
     * @Description 报告概况--历史排名变化
     * @Param [studentid]
     * @Return java.util.List<java.util.Map>
     * @Author wn
     * @Date 2022/8/25 14:35
     **/
    public List<Map> listStudentRankHistory(Integer studentid) {
        return eRstudentMapper.listStudentExamHistory(studentid, "zf");
    }

    /**
     * @Description 学科分析--名次和历史排名
     * @Param [examid, studentid, subjectid]
     * @Return java.util.Map
     * @Author wn
     * @Date 2022/8/25 15:31
     **/
    public Map getSubjectScoreByStudentid(Integer examid, Integer studentid, String subjectid) {
        Map rtnmap = eRstudentMapper.getSubjectScoreByStudentid(examid, studentid, subjectid);
        if (rtnmap != null && rtnmap.size() > 0) {
            Integer classid = Integer.parseInt(rtnmap.get("classid").toString());
            Map gcnum = eRstudentMapper.getStudentNumByExamid(examid, classid);
            rtnmap.put("classnum", gcnum.get("classnum").toString());
            rtnmap.put("gradenum", gcnum.get("gradenum").toString());

            //历史名次
            List<Map> shlist = eRstudentMapper.listStudentExamHistory(studentid, subjectid);
            rtnmap.put("historys", shlist);
        }
        return rtnmap;
    }

    /**
     * @Description 学科分析--知识点分析
     * @Param [examid, studentid, subjectid]
     * @Return java.util.List<java.util.Map>
     * @Author wn
     * @Date 2022/8/25 15:41
     **/
    public List<Map> listSubjectPoints(Integer examid, Integer studentid, String subjectid) {
        return eRstudentMapper.listSubjectPoints(examid, studentid, subjectid);
    }

    /**
     * @Description 学科分析--难易度分析
     * @Param [examid, studentid, subjectid]
     * @Return java.util.List<java.util.Map>
     * @Author wn
     * @Date 2022/8/25 15:52
     **/
    public List<Map> listSubjectComplexitys(Integer examid, Integer studentid, String subjectid) {
        return eRstudentMapper.listSubjectComplexitys(examid, studentid, subjectid);
    }

    /**
     * @Description 学科分析--题型分析
     * @Param [examid, studentid, subjectid]
     * @Return java.util.List<java.util.Map>
     * @Author wn
     * @Date 2022/8/25 16:21
     **/
    public List<Map> listSubjectQtypes(Integer examid, Integer studentid, String subjectid) {
        return eRstudentMapper.listSubjectQtypes(examid, studentid, subjectid);
    }

    /**
     * @Description
     * @Param [examid, studentid, subjectid]
     * @Return java.util.List<java.util.Map>
     * @Author wn
     * @Date 2022/8/25 16:54
     **/
    public List<Map> listSubjectQuestions(Integer examid, Integer studentid, String subjectid) {
        List<Map> list = eRstudentMapper.listSubjectQuestions(examid, studentid, subjectid);
        for (Map m : list) {
            Double stuscore = Double.parseDouble(m.get("stuscore").toString());
            Double qscore = Double.parseDouble(m.get("qscore").toString());
            Double stusrate = N_Utils.getDoubleDivideAndMulitiply(stuscore, qscore);
            m.put("stusrate", stusrate);
        }
        return list;
    }

    /**
     * @Description 试卷分析
     * @Param [examid, studentid, subjectid]
     * @Return java.util.List<java.util.Map>
     * @Author wn
     * @Date 2022/8/25 17:50
     **/
    public Map getStudentPaperQuestions(Integer examid, Integer studentid, String subjectid) {
        Map map = ePaperMapper.getPaperByExamidAndSubjectid(examid, subjectid);
        Integer ptype = Integer.parseInt(map.get("ptype").toString());
        Integer epid = Integer.parseInt(map.get("epid").toString());
        List<Map> list = null;
        if (ptype == 1) {
            list = eRstudentMapper.listStudentQuestionsTk(epid, studentid);
        } else {
            list = eRstudentMapper.listStudentQuestionsFj(epid, studentid);
            List<Map> files = ePaperFileMapper.listPaperFiles(epid);
            map.put("files", files);
        }
        map.put("questions", list);
        return map;
    }

    //处理母子题
    private List<Map> setPaperQuestionForTK(List<Map> list){
        List<Map> rtnlist = new ArrayList<>();
        List<String> questiopids = new ArrayList<>();
        for(Map q : list){
            Integer qlevel = Integer.parseInt(q.get("qlevel").toString());
            if(qlevel == 1){
                //代表单题
                Map map = new HashMap();
                map.put("epsqid",Integer.parseInt(q.get("epsqid").toString()));
                map.put("eptqid",Integer.parseInt(q.get("eptqid").toString()));
                map.put("ctype",Integer.parseInt(q.get("ctype").toString()));
                map.put("qn",q.get("qn").toString());
                map.put("qorder",Integer.parseInt(q.get("qorder").toString()));
                map.put("qscore",Double.parseDouble(q.get("qscore").toString()));
                map.put("stuscore",Double.parseDouble(q.get("stuscore").toString()));
                map.put("answered",Integer.parseInt(q.get("answered").toString()));
                map.put("answertype",Integer.parseInt(q.get("answertype").toString()));
                map.put("stuanswer",q.get("stuanswer").toString());
                map.put("stuanswertxt",q.get("stuanswertxt").toString());
                map.put("qstem",q.get("qstem").toString());
                map.put("qoption",q.get("qoption").toString());
                map.put("qanswer",q.get("qanswer").toString());
                map.put("qanalyze",q.get("qanalyze").toString());
                map.put("hashear",Integer.parseInt(q.get("hashear").toString()));
                map.put("hearfile",q.get("hearfile").toString());
                map.put("qlevel",qlevel);
                map.put("classavgscore",Integer.parseInt(q.get("classavgscore").toString()));
                map.put("gradeavgscore",Integer.parseInt(q.get("gradeavgscore").toString()));
                if(q.get("points")!=null){
                    List<Map> points = (List<Map>) q.get("points");
                    map.put("points",points);
                }
                rtnlist.add(map);
            }else{
                //母题
                String questionpid = q.get("questionpid").toString();
                if(!questiopids.contains(questionpid)){
                    questiopids.add(questionpid);
                    //获取所有试题
                    List<Map> sonqueslist = list.stream().filter(s -> s.get("questionpid").toString().equals(questionpid)).collect(Collectors.toList());
                    Map mq = new HashMap();
                    mq.put("questionid",questionpid);
                    mq.put("qlevel",2);
                    mq.put("qorder",Integer.parseInt(q.get("qorder").toString()));
                    mq.put("ctype",Integer.parseInt(q.get("mctype").toString()));
                    mq.put("qstem",q.get("mqstem").toString());
                    mq.put("qoption",q.get("mqoption").toString());
                    mq.put("qanswer",q.get("mqanswer").toString());
                    mq.put("qanalyze",q.get("mqanalyze").toString());
                    mq.put("hashear",Integer.parseInt(q.get("mhashear").toString()));
                    mq.put("hearfile",q.get("mhearfile").toString());
                    if(q.get("points")!=null){
                        List<Map> points = (List<Map>) q.get("points");
                        mq.put("points",points);
                    }
                    mq.put("sonques",sonqueslist);
                    rtnlist.add(mq);
                }
            }
        }
        return rtnlist;
    }
}
